/*
 * DS1337RTC.h - library for DS1337 RTC

  Copyright (C) 2013 I.Kato @ BS21Lab.
  This library is intended to be uses with Arduino Time.h library functions

  The library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

   9/oct/2013 - Initial release by I.Kato @ BS21Lab (based on DS1307RTC Library.)
  20/oct/2013 - powerOffwithoutWaking() added by I.Kato @ BS21Lab
 */

#if ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

#include <Wire.h>
#include "DS1337RTC.h"


#ifndef __arm__
// Arduino Uno R3 等の時
#define DS1337_Wire  Wire
#else // __arm__
// Arduino Due の時
#define DS1337_Wire  Wire1
#endif // __arm__

//////////////////////////////////////////
// constants of Wiring core API

#define Wire_beginT(addr)  DS1337_Wire.beginTransmission(addr)
#define Wire_endT()        DS1337_Wire.endTransmission()

#define Wire_reqFrom(addr,len)  DS1337_Wire.requestFrom((addr),(len))

#if defined(ARDUINO) && ARDUINO >= 100
#define Wire_read()      DS1337_Wire.read()
#define Wire_write(val)  DS1337_Wire.write((uint8_t)(val))
#else
#define Wire_read()      DS1337_Wire.receive()
#define Wire_write(val)  DS1337_Wire.send(val)
#endif



//////////////////////////////////////////
// constants & macros

#define DS1337_TWI_ADDR      0x68   // DS1337 の TWI アドレス(変更不能)

#define DS1337_TIME_REG      0x00   // 現在の日時 (秒/分/時/曜日/日/月/年の順)

#define DS1337_ALARM1_REG    0x07   // アラーム1：TPS2421 の ~EN に接続
#define DS1337_ALARM1_MASK   B10000000  // 0=設定値でアラームON, 1=常にアラームON
#define DS1337_ALARM1_WDAY   B01000000  // 0=日(day), 1=曜日(DOW)

#define DS1337_ALARM2_REG    0x0B   // アラーム2：未使用
#define DS1337_ALARM2_MASK   B10000000  // 同上
#define DS1337_ALARM2_WDAY   B01000000  // 同上

#define DS1337_CTRL_REG      0x0E   // 制御レジスタ
#define DS1337_CTRL_EOSC     B10000000  // 0:オシレータ出力開始(default), 1:オシレータ出力停止
#define DS1337_CTRL_RS2      B00010000  // オシレータ出力の Rate 設定(2bit)  00:1Hz, 01:4096Hz, 10:8192Hz, 11:32768Hz(default)
#define DS1337_CTRL_RS1      B00001000  // 同上
#define DS1337_CTRL_INTCN    B00000100  // 0:アラーム2割込みの時も~INTAピンに出力する(default), 1:アラーム2割込みの時、~INTBピンに出力する
#define DS1337_CTRL_A2IE     B00000010  // 1:アラーム2割込みEnable, 0:Disable(default)
#define DS1337_CTRL_A1IE     B00000001  // 1:アラーム1割込みEnable, 0:Disable(default)

#define DS1337_STATUS_REG    0x0F   // 状態レジスタ
#define DS1337_STATUS_OSF    B10000000  // オシレータ停止Flag   1:オシレータ出力停止中, 0:オシレータ出力中
#define DS1337_STATUS_A2F    B00000010  // アラーム2割込みFlag  1:割込み有った, 0:割込み未だ
#define DS1337_STATUS_A1F    B00000001  // アラーム1割込みFlag  1:割込み有った, 0:割込み未だ


// ＜＜ 自分用の備忘録 ＞＞

// ~INTA ピンと ~INTB ピンは負論理。アラーム発生時に L になる。オープン・ドレインなのに耐圧 +5.5V まで。
// アラーム1 は常に ~INTA ピンに出力されるが、アラーム2 は SQW/~INTB ピンへの出力か、~INTA ピンへの出力(アラーム1とNORされる)のいずれかを選択できる。
// アラーム2 が ~INTA ピンへの出力になっている場合、SQW/~INTB ピンからは 32.768kHz そのものか、分周した矩形波を出力できる。
// DS1337 は初期状態では、~INTA ピンはアラーム1とアラーム2 の NOR出力(ただしA1IE/A2IEはOFF)、SQW/~INTB ピンからは 32.768kHz が出力enableされている。

// アラーム1 は、日or曜日/時/分/秒の４レジスタを持ち、この４つ全部がONになった時に初めてアラーム出力される。
// 各レジスタは MSB=1 の時は常にONになるので、例えば、分/秒のみ設定値をセットし、日or曜日/時は MSB=1 にしておけば、
// 日or曜日/時に関係なく、RTC の分/秒がそれぞれの設定値（例えば12分34秒など）に一致した時に、アラーム出力ON になる。

// アラーム2 もほぼ同様。ただし、秒レジスタが無く、日or曜日/時/分の３レジスタのみ。

// アラーム1 の ~INTA ピンは TPS2412 の ~EN に直結されており、アラーム発生時に L になる事で、TPS2412 を ON させる。
// アラーム1 に新しい日時をセットし、powerOff関数で A1IE を ON にすると、~INTA ピンは H になるので、TPS2412 は OFF になる。

// アラーム2 の SQW/~INTB ピンは Arduino の 2/3/7ピンに直結できるので、アラーム発生時に Arduino に割り込みをかける事が可能。
// ただし、現ライブラリではソフト的に未だ対応していない（Time ライブラリ付属の TimeAlarms ライブラリで似た事が行えそうなので、必要性は薄いと思う）。
// 対応する時は、Uno と Leonardo で割込み処理を変えないといけない(めんどくさいなぁ)。
//                        2ピン   3ピン   7ピン
//     Uno     (m328P)    INT0    INT1    (AIN1)
//     Leonardo(m32U4)    (SDA)   (SCL)   INT6
// ※ Uno では 2/3ピンのどちらかを INT0/1 として使える。7ピンでも出来なくは無いが、ピンチェンジ割込みなので、割込み処理がやや面倒で応答が遅くなる。
// ※ Leonardo では 2/3ピンは TWI に使われるので、必然的に7ピンに割り付けられている INT6 を使うしか、手が無い。

// 上記の通り、アラーム2 も ~INTA ピンに出力できるので、"目覚めよ!xxduino!シールド" 的には、本当は２つのアラーム日時で目覚めさせる事が可能。
// （例えば、毎時00分と 30分に設定しておけば、30分おきに目覚めさせられる）
// ただし、現ライブラリではソフト的に対応していない（setWakeupTimeEveryXxx関数がアラーム1/2それぞれで必要になり、分かりづらくなってしまう為）。
// ※ 現ライブラリでも、目覚めるたびに次のアラーム日時を設定するようにすれば、30分おきに目覚めさせる事が可能。

// アラーム1 で目覚めた時は、A1F フラグが ON になっている。逆に、基板上の強制ON用SWで TPS2412を強制ON させた時や、USB 給電した時は、A1F フラグは OFF のまま。
// よって、A1F フラグを見れば、アラームで目覚めたのか、そうでないか、が区別できる。



//////////////////////////////////////////
// グローバル変数 global variables

DS1337RTC RTC = DS1337RTC(); // create an instance for the user



//////////////////////////////////////////
// DS1337RTC class

DS1337RTC::DS1337RTC()
{
  DS1337_Wire.begin();  // TWI の初期化
}

// PUBLIC FUNCTIONS

//////////////////////////////////////////
// 現在の日時

// Aquire data from buffer and convert to time_t
time_t DS1337RTC::getCurrTime (void)
{
  tmElements_t tm;
  getCurrTime(tm);
  return(makeTime(tm));
}

// Aquire data from the RTC chip in BCD format
void DS1337RTC::getCurrTime (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);
  Wire_write(DS1337_TIME_REG);
  Wire_endT();

  // request the 7 data fields   (secs, min, hr, dow, date, mth, yr)
  Wire_reqFrom(DS1337_TWI_ADDR, 7);

  tm.Second = bcd2dec(Wire_read());
  tm.Minute = bcd2dec(Wire_read());
  tm.Hour   = bcd2dec(Wire_read());       // 24時間formatのみ。12時間formatではおかしくなる
  tm.Wday   = Wire_read();                // DS1337RTC_DOW_xxx のいずれか
  tm.Day    = bcd2dec(Wire_read());
  tm.Month  = bcd2dec(Wire_read() & 0x1F);  // Centuryビットは無視
  tm.Year   = y2kYearToTm((bcd2dec(Wire_read())));  // 1970年からの差
}


//////////////////////////////////////////
// 日時を設定する

void DS1337RTC::setCurrTime (time_t t)
{
  tmElements_t tm;
  breakTime(t, tm);
  setCurrTime(tm);
}

void DS1337RTC::setCurrTime (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_TIME_REG); // set register pointer

  Wire_write(dec2bcd(tm.Second));
  Wire_write(dec2bcd(tm.Minute));
  Wire_write(dec2bcd(tm.Hour));      // sets 24 hour format
  Wire_write(tm.Wday);               // DS1337RTC_DOW_xxx のいずれか
  Wire_write(dec2bcd(tm.Day));
  Wire_write(dec2bcd(tm.Month));
  Wire_write(dec2bcd(tmYearToY2k(tm.Year)));  // 西暦年の下2桁: lower 2 digits of year

  Wire_endT();
}

void DS1337RTC::setCurrTime (uint8_t hr, uint8_t min, uint8_t sec, uint8_t day, uint8_t month, uint16_t yr) // 引数の並びに注意
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_TIME_REG); // set register pointer

  Wire_write(dec2bcd(sec));
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));      // sets 24 hour format
  Wire_write(calcDayOfWeek(day, month, yr));  // 年月日から曜日を算出
  Wire_write(dec2bcd(day));
  Wire_write(dec2bcd(month));
  Wire_write(dec2bcd(yr - 2000)); // 西暦年の下2桁: lower 2 digits of year

  Wire_endT();
}


//////////////////////////////////////////
// Wakeup の日時

// 設定されている次回の wakeup 日時と、wakeup の方法 (DS1337RTC_WAKEUP_EVERY_XXX のいずれか) が返されます
uint8_t DS1337RTC::getWakeupTime (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);
  Wire_write(DS1337_ALARM1_REG);
  Wire_endT();

  Wire_reqFrom(DS1337_TWI_ADDR, 4);

  uint8_t sec = Wire_read();
  uint8_t min = Wire_read();
  uint8_t hr  = Wire_read();
  uint8_t day = Wire_read();

  uint8_t type = DS1337RTC_WAKEUP_EVERY_SEC;

  if ((sec & DS1337_ALARM1_MASK) == 0) { // 秒がセットされている
    type      = DS1337RTC_WAKEUP_EVERY_MIN;
    tm.Second = bcd2dec(sec & 0x7F);
  } else {
    tm.Second = 0;
  }

  if ((min & DS1337_ALARM1_MASK) == 0) { // 分がセットされている
    type      = DS1337RTC_WAKEUP_EVERY_HOUR;
    tm.Minute = bcd2dec(min & 0x7F);
  } else {
    tm.Minute = 0;
  }

  if ((hr & DS1337_ALARM1_MASK) == 0) { // 時がセットされている
    type    = DS1337RTC_WAKEUP_EVERY_DAY;
    tm.Hour = bcd2dec(hr & 0x7F);    // 24時間formatのみ。12時間formatではおかしくなる
  } else {
    tm.Hour = 0;
  }

  if ((day & DS1337_ALARM1_MASK) == 0) { // 日がセットされている
    if (day & DS1337_ALARM1_WDAY) {  // 曜日がセットされている
      type    = DS1337RTC_WAKEUP_EVERY_WEEK;
      tm.Wday = day & 0x07;  // DS1337RTC_DOW_xxx のいずれか
      tm.Day  = 0;
    } else { // 曜日ではなかった
      type    = DS1337RTC_WAKEUP_EVERY_MONTH;
      tm.Wday = 0;
      tm.Day  = bcd2dec(day & 0x3F);
    }
  } else {
    tm.Wday = 0;
    tm.Day  = 0;
  }

  tm.Month  = 0;
  tm.Year   = 0;

  return type;
}


// 毎分、指定の秒に wakeup させます。
void DS1337RTC::setWakeupTimeEveryMin (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(tm.Second));
  Wire_write(DS1337_ALARM1_MASK);  // 分は常にON：0～59分のいずれでもONになる
  Wire_write(DS1337_ALARM1_MASK);  // 時は常にON：0～23時のいずれでもONになる
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON：1～31日のいずれでもONになる

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryMin (void)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(0);  // 0秒固定
  Wire_write(DS1337_ALARM1_MASK);  // 分は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 時は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryMin (uint8_t sec)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(sec));
  Wire_write(DS1337_ALARM1_MASK);  // 分は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 時は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

// 毎時、指定の分/秒に wakeup させます。
void DS1337RTC::setWakeupTimeEveryHour (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(tm.Second));
  Wire_write(dec2bcd(tm.Minute));
  Wire_write(DS1337_ALARM1_MASK);  // 時は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryHour (uint8_t min)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(0);  // 0秒固定
  Wire_write(dec2bcd(min));
  Wire_write(DS1337_ALARM1_MASK);  // 時は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryHour (uint8_t min, uint8_t sec)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(sec));
  Wire_write(dec2bcd(min));
  Wire_write(DS1337_ALARM1_MASK);  // 時は常にON
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

// 毎日、指定の時/分/秒に wakeup させます。
void DS1337RTC::setWakeupTimeEveryDay (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(tm.Second));
  Wire_write(dec2bcd(tm.Minute));
  Wire_write(dec2bcd(tm.Hour));    // sets 24 hour format
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryDay (uint8_t hr, uint8_t min)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(0);  // 0秒固定
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));         // sets 24 hour format
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryDay (uint8_t hr, uint8_t min, uint8_t sec)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(sec));
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));         // sets 24 hour format
  Wire_write(DS1337_ALARM1_MASK);  // 日は常にON

  Wire_endT();
}

// 毎週、指定の曜日/時/分/秒に wakeup させます。
void DS1337RTC::setWakeupTimeEveryWeek (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(tm.Second));
  Wire_write(dec2bcd(tm.Minute));
  Wire_write(dec2bcd(tm.Hour));      // sets 24 hour format
  Wire_write(DS1337_ALARM1_WDAY | tm.Wday);  // day of week, sunday is day 1

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryWeek (uint8_t dayOfWeek, uint8_t hr, uint8_t min) // dayOfWeek = DS1337RTC_DOW_xxx のいずれか
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(0);  // 0秒固定
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));      // sets 24 hour format
  Wire_write(DS1337_ALARM1_WDAY | dayOfWeek);  // day of week, sunday is day 1

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryWeek (uint8_t dayOfWeek, uint8_t hr, uint8_t min, uint8_t sec) // dayOfWeek = DS1337RTC_DOW_xxx のいずれか
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(sec));
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));      // sets 24 hour format
  Wire_write(DS1337_ALARM1_WDAY | dayOfWeek);  // day of week, sunday is day 1

  Wire_endT();
}

// 毎月、指定の日/時/分/秒に wakeup させます。
void DS1337RTC::setWakeupTimeEveryMonth (tmElements_t &tm)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(tm.Second));
  Wire_write(dec2bcd(tm.Minute));
  Wire_write(dec2bcd(tm.Hour));      // sets 24 hour format
  Wire_write(dec2bcd(tm.Day));

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryMonth (uint8_t day, uint8_t hr, uint8_t min) // 引数の並びに注意：日(day)から指定します
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(0);  // 0秒固定
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));      // sets 24 hour format
  Wire_write(dec2bcd(day));

  Wire_endT();
}

void DS1337RTC::setWakeupTimeEveryMonth (uint8_t day, uint8_t hr, uint8_t min, uint8_t sec) // 引数の並びに注意：日(day)から指定します
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(DS1337_ALARM1_REG); // set register pointer

  Wire_write(dec2bcd(sec));
  Wire_write(dec2bcd(min));
  Wire_write(dec2bcd(hr));      // sets 24 hour format
  Wire_write(dec2bcd(day));

  Wire_endT();
}

// wakeup フラグの状態を取得します。
// 指定の Wakeup 時間で電源 ON した時は 0 以外に、他の方法で電源 ON した時は 0 になります。
uint8_t DS1337RTC::getWakeupFlag (void)
{
  uint8_t val = readReg (DS1337_STATUS_REG);
  return (val & DS1337_STATUS_A1F);
}

// wakeup フラグを強制的にクリアします。通常、使う事は無いでしょう。
void DS1337RTC::clearWakeupFlag (void)
{
  uint8_t val = readReg (DS1337_STATUS_REG);
  val &= ~DS1337_STATUS_A1F;
  writeReg (DS1337_STATUS_REG, val);
}


// 電源を OFF にします。目覚めよ！xxduino！シールドからの９Ｖ供給が停止します。
// USB 接続されていない場合、直ぐに Arduino が停止しますので、直前で、各種データを保存or送信し、次に目覚める Wakeup 日時を設定してから、これを呼び出してください。
bool DS1337RTC::powerOff (void)
{
  clearWakeupFlag(); // A1F フラグを強制的にクリアする。これが立ちっぱなしだと A1IE を ON にしても alarm1 が有効にならない模様

  uint8_t val = readReg (DS1337_CTRL_REG);
  val |= DS1337_CTRL_INTCN | DS1337_CTRL_A1IE;  // A1IE フラグを ON にする ⇒ alarm1 を有効にする
  writeReg (DS1337_CTRL_REG, val);

  return true;
}


// 電源を OFF にします。目覚めよ！xxduino！シールドからの９Ｖ供給が停止します。Wakeup 日時も無視されますので、USB 接続するまで RTC/SRAM 以外は眠り続けます。
// USB 接続されていない場合、直ぐに Arduino が停止しますので、直前で、各種データを保存or送信してから、これを呼び出してください。
// 目覚めよ！xxduino！シールドのアラーム１出力を強制 OFF する時も、この関数を呼び出してください。
bool DS1337RTC::powerOffWithoutWaking (void)
{
  clearWakeupFlag(); // A1F フラグを強制的にクリアする。

  uint8_t val = readReg (DS1337_CTRL_REG);
  val &= ~DS1337_CTRL_A1IE;  // A1IE フラグを OFF にする ⇒ alarm1 を無効にする
  writeReg (DS1337_CTRL_REG, val);

  return true;
}



// ツェラーの公式の月の部分の計算済みテーブル: -1 しているのは d-1 の分を先取りしているため
// ※ SRAM をケチるため、プログラムメモリに格納する
const uint8_t ZellerMonthTable[] PROGMEM = {
  (uint8_t)(26 * (13 + 1) / 10) - 1,  // Jan
  (uint8_t)(26 * (14 + 1) / 10) - 1,  // Feb
  (uint8_t)(26 * ( 3 + 1) / 10) - 1,  // Mar
  (uint8_t)(26 * ( 4 + 1) / 10) - 1,  // Apr
  (uint8_t)(26 * ( 5 + 1) / 10) - 1,  // May
  (uint8_t)(26 * ( 6 + 1) / 10) - 1,  // Jun
  (uint8_t)(26 * ( 7 + 1) / 10) - 1,  // Jul
  (uint8_t)(26 * ( 8 + 1) / 10) - 1,  // Aug
  (uint8_t)(26 * ( 9 + 1) / 10) - 1,  // Sep
  (uint8_t)(26 * (10 + 1) / 10) - 1,  // Oct
  (uint8_t)(26 * (11 + 1) / 10) - 1,  // Nov
  (uint8_t)(26 * (12 + 1) / 10) - 1,  // Dec
};

// 曜日の計算: calculating day of week   1:日(Sun), 2:月(Mon), ..., 7:土(Sat)
// 注意:2000/3/1～2100/2/28 の範囲でしか正しい値になりません。
// ツェラーの公式(Zeller's congruence)の簡略版
uint8_t DS1337RTC::calcDayOfWeek (uint8_t day, uint8_t month, uint16_t yr)
{
  uint8_t C = 20;  // 西暦年の上2桁: 20 で固定  : upper 2 digits of year == 20
  uint8_t Y = (uint8_t)(yr - 2000);  // 西暦年の下2桁: lower 2 digits of year
  if (month <= 2) {
    // 1月,2月は前の年の13月,14月とみなす
    Y--;  // 2000/1/1～2000/2/29 の間は範囲外としたので、これで構わない(以下の計算が面倒になるので 1999 年にはしない)。
  }
  // 符号無し 8bit の範囲内で計算できるように工夫する
  uint8_t h = ((5 * C) + (C / 4)) % 7;  // 5*C + floor(C/4) : 分配法則より、ここで % 7 しても問題無い : 定数(0)になるのでコンパイラが賢ければ毎回計算しなくて済むはず
  h += Y + (Y >> 2);  // Y + floor(Y/4)  :  0～123 の範囲
  h += pgm_read_byte(&ZellerMonthTable[month - 1]);  // floor(26*(m+1) / 10) : 9～38 の範囲
  h += day;  // 本来は d-1 を加算だが、-1 の分は m の時点で先にしてあるので、d をそのまま加算するだけでOK : 1～31 の範囲
  // ここまでで 0～192 の範囲に納まるので、十分 8bit の範囲に納まる
#if 1
  // 剰余は計算量が多いので最後に１回だけ実施する
  h %= 7;
#else
  // 剰余の計算 : 引き算だけで h %= 7 と同じ処理をさせる : 高速だがコンパイルさせると意外とコードが大きくなってしまったので、採用しない事にした
  if (h >= 7 * 16) h -= 7 * 16;
  if (h >= 7 *  8) h -= 7 *  8;
  if (h >= 7 *  4) h -= 7 *  4;
  if (h >= 7 *  2) h -= 7 *  2;
  if (h >= 7 *  1) h -= 7 *  1;
#endif
  return h + DS1337RTC_DOW_SUN; // 剰余の結果、0:日, 1:月, ..., 6:土 になるので、+1 して、DS1337 仕様に合わせる
}


//////////////////////////////////////////
// PRIVATE FUNCTIONS

// Convert Decimal to Binary Coded Decimal (BCD)
uint8_t DS1337RTC::dec2bcd (uint8_t num)
{
  return ((num / 10 * 16) + (num % 10));
}

// Convert Binary Coded Decimal (BCD) to Decimal
uint8_t DS1337RTC::bcd2dec (uint8_t num)
{
  return ((num / 16 * 10) + (num % 16));
}


// DS1337 のレジスタの値を読み出す
uint8_t DS1337RTC::readReg (uint8_t reg_no)
{
  Wire_beginT(DS1337_TWI_ADDR);
  Wire_write(reg_no);
  Wire_endT();

  Wire_reqFrom(DS1337_TWI_ADDR, 1);

  return Wire_read();
}

// DS1337 のレジスタに新しい値をセットする
void DS1337RTC::writeReg (uint8_t reg_no, uint8_t val)
{
  Wire_beginT(DS1337_TWI_ADDR);

  Wire_write(reg_no); // set register pointer

  Wire_write(val);

  Wire_endT();
}

