/*
 * DS1337RTC.h - library for DS1337 RTC
 *
 * Copyright (C) 2013 I.Kato @ BS21Lab.
 * This library is intended to be uses with Arduino Time.h library functions
 * 本ライブラリは、Arduino 用 Time ライブラリと組み合わせて使用します。
 *
 * The library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  9/oct/2013 - Initial release by I.Kato @ BS21Lab (based on DS1307RTC Library.)
 * 20/oct/2013 - powerOffwithoutWaking() added by I.Kato @ BS21Lab
 *
 */

#ifndef DS1337RTC_h
#define DS1337RTC_h

#include <Time.h>

// DS1337 と TWI(i2c) 通信をします
// Time ライブラリと組み合わせて使用します（DS1307RTC ライブラリと同じ方法）。
// 注意1：DS1307, DS3231 とは i2c アドレスがかぶる（共に 0x68）ので、共存できません。(もっとも RTC は１個あれば十分でしょうが、念の為。)
// 注意2：tmElements_t は７バイト消費しますが、直感的な構造体なので使いやすいです。
// 注意3：tmElements_t.Year は 1970年からの offset 値なので、見かけ上、西暦年の下2桁＋30 されているように見えます。なので -30 するか、または +1970 してください。
// 注意4：time_t は 1970/1/1 00:00:00 からの経過秒です。４バイトとコンパクトですが、直感的ではないので、やや使いづらいです。
// 注意5：Time ライブラリに合わせる為、setCurrTime 関数の引数は、時/分/秒/日/月/年、と言う変則的な並びになっています。
//        一方、setWakeupTimeEveryMonth 関数の引数は、日/時/分/秒、と普通の並びなので、両者の違いに注意してください。
// 注意6：DS1337 自体、１日当たり0～±2秒ほどズレが生じます。月にすると0秒～±1分ほどのズレになります。気温によってもズレが増減します。よって、秒の単位は目安と考えてください。
//        １ヶ月以上運用する場合は、分の単位でズレますので、何らかの方法（例えばWiFi等）で定期的に時間校正してやる必要があります。
// 注意7：現 version では、DS1337 の Alarm2 (INTB) は使用していません。
// 注意8：ライブラリ名が DS1307RTC と非常に良く似ている（一文字違い）ので、混同しないように注意してください。


// wakeup の方法
#define DS1337RTC_WAKEUP_EVERY_SEC    1  // 毎秒：使用しません(Arduinoの起動そのものに数秒かかる為、毎秒起動は逆立ちしても不可能です)。ただし、DS1337の初期状態によっては、この設定になっている場合がありえます。
#define DS1337RTC_WAKEUP_EVERY_MIN    2  // 毎分：指定の秒ごとに wakeup する
#define DS1337RTC_WAKEUP_EVERY_HOUR   3  // 毎時：指定の分/秒ごとに wakeup する
#define DS1337RTC_WAKEUP_EVERY_DAY    4  // 毎日：指定の時/分/秒ごとに wakeup する
#define DS1337RTC_WAKEUP_EVERY_WEEK   5  // 毎週：指定の曜日/時/分/秒ごとに wakeup する
#define DS1337RTC_WAKEUP_EVERY_MONTH  6  // 毎月：指定の日/時/分/秒ごとに wakeup する

// 曜日：DS1337では 1～7 の範囲になります。Time ライブラリもこれと同じです。
//       DS1337 以外の RTC では違う値になってたりするので要注意です。
#define DS1337RTC_DOW_SUN   1  // 日
#define DS1337RTC_DOW_MON   2  // 月
#define DS1337RTC_DOW_TUE   3  // 火
#define DS1337RTC_DOW_WED   4  // 水
#define DS1337RTC_DOW_THU   5  // 木
#define DS1337RTC_DOW_FRI   6  // 金
#define DS1337RTC_DOW_SAT   7  // 土


// library interface description
class DS1337RTC
{
public:
  DS1337RTC();

public:
  static time_t getCurrTime (void);   // 現在の日時を取得
  static void   getCurrTime (tmElements_t &tm);  // tm.Year の注意事項は上記参照
public:
  static void setCurrTime (time_t t);  // 日時を設定
  static void setCurrTime (tmElements_t &tm);  // tm.Year の注意事項は上記参照
  static void setCurrTime (uint8_t hr, uint8_t min, uint8_t sec, uint8_t day, uint8_t month, uint16_t yr); // 引数の並びに注意：yr = 2000～2099

public:
  static uint8_t getWakeupTime (tmElements_t &tm);  // 設定されている次回の wakeup 日時と、wakeup の方法 (DS1337RTC_WAKEUP_EVERY_XXX のいずれか) が返されます
public:
  // Wakeup 日時の設定：Alarm1を使用
  // 毎分、指定の秒に wakeup させます。
  static void setWakeupTimeEveryMin   (tmElements_t &tm);
  static void setWakeupTimeEveryMin   (void);    // 0秒指定
  static void setWakeupTimeEveryMin   (uint8_t sec);
  // 毎時、指定の分/秒に wakeup させます。
  static void setWakeupTimeEveryHour  (tmElements_t &tm);
  static void setWakeupTimeEveryHour  (uint8_t min);               // min分0秒指定
  static void setWakeupTimeEveryHour  (uint8_t min, uint8_t sec);  // min分sec秒指定
  // 毎日、指定の時/分/秒に wakeup させます。
  static void setWakeupTimeEveryDay   (tmElements_t &tm);
  static void setWakeupTimeEveryDay   (uint8_t hr, uint8_t min);               // hr時min分0秒指定
  static void setWakeupTimeEveryDay   (uint8_t hr, uint8_t min, uint8_t sec);  // hr時min分sec秒指定
  // 毎週、指定の曜日/時/分/秒に wakeup させます。
  static void setWakeupTimeEveryWeek  (tmElements_t &tm);
  static void setWakeupTimeEveryWeek  (uint8_t dayOfWeek, uint8_t hr, uint8_t min); // dayOfWeek = DS1337RTC_DOW_xxx のいずれか
  static void setWakeupTimeEveryWeek  (uint8_t dayOfWeek, uint8_t hr, uint8_t min, uint8_t sec); // 同上
  // 毎月、指定の日/時/分/秒に wakeup させます。
  static void setWakeupTimeEveryMonth (tmElements_t &tm);
  static void setWakeupTimeEveryMonth (uint8_t day, uint8_t hr, uint8_t min);  // 引数の並びに注意：日(day)から指定します
  static void setWakeupTimeEveryMonth (uint8_t day, uint8_t hr, uint8_t min, uint8_t sec);  // 同上
public:
  static uint8_t getWakeupFlag   (void);   // wakeup フラグの状態を取得します。指定の Wakeup 時間で電源 ON した時は 0 以外に、他の方法で電源 ON した時は 0 になります。
  static void    clearWakeupFlag (void);   // wakeup フラグを強制的にクリアします。通常、使う事は無いでしょう。

public:
  // 電源を OFF にします。目覚めよ！xxduino！シールドからの９Ｖ供給が停止します。
  // USB 接続されていない場合、直ぐに Arduino が停止しますので、直前で、各種データを保存or送信し、次に目覚める Wakeup 日時を設定してから、これを呼び出してください。
  static bool powerOff (void);

public:
  // 電源を OFF にします。目覚めよ！xxduino！シールドからの９Ｖ供給が停止します。Wakeup 日時も無視されますので、USB 接続するまで RTC/SRAM 以外は眠り続けます。
  // USB 接続されていない場合、直ぐに Arduino が停止しますので、直前で、各種データを保存or送信してから、これを呼び出してください。
  // 目覚めよ！xxduino！シールドのアラーム１出力を強制 OFF する時も、この関数を呼び出してください。
  static bool powerOffWithoutWaking (void);

public:
  // 曜日の計算   1:日(Sun), 2:月(Mon), ..., 7:土(Sat)    注意:2000/3/1～2100/2/28 の範囲でしか正しい値になりません。
  static uint8_t calcDayOfWeek (uint8_t day, uint8_t month, uint16_t yr);

private:  // 内部使用
  static uint8_t dec2bcd (uint8_t num);
  static uint8_t bcd2dec (uint8_t num);

  static uint8_t readReg  (uint8_t reg_no);
  static void    writeReg (uint8_t reg_no, uint8_t val);
};


// インスタンス生成
extern DS1337RTC RTC;

#endif

