/*
 * TimeRTCSet for DS1337
 * example code illustrating Time library with Real Time Clock.
 *
 * RTC clock is set in response to serial port time message
 *
 * DS1337 に日時を設定するスケッチ例
 *
 * 使い方：
 *   目覚めよ！xxduino！シールドを接続した Arduino に、このスケッチを転送します。
 *   転送後、シリアルモニタを開き、tmNewTime で設定した日時になった瞬間に、アルファベットのSを送れば
 *   目覚めよ！xxduino！シールドの RTC である DS1337 に、日時が設定できます。
 *   当然ながら DS1337 はバッテリーバックアップされている必要が有ります。
 *
 * Copyright (C) 2013 I.Kato @ BS21Lab.
 *
 * The sketch is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This sketch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this sketch; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <Time.h>
#include <Wire.h>
#include <DS1337RTC.h>  // a basic DS1337 library that returns time as a time_t


// RTC:DS1337 にセットしたい日時に書き換えてください。
tmElements_t tmNewTime = {
  0,    // sec:  0～59
  0,    // min:  0～59
  14,   // hour: 0～23
  5,    // day of week, 1:sun(日), 2:mon(月), 3:tue(火), 4:wed(水), 5:thu(木), 6:fri(金), 7:sat(土)
  17,   // day:   1～31
  10,   // month: 1～12
  2013 - 1970,  // year, offset from 1970;   1970年からの差を指定する
};

void setup()
{
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  // Time ライブラリに DS1337 の getCurrTime 関数を渡します。以降、Time ライブラリ内で DS1337 の日時が取得されます。
  setSyncProvider(RTC.getCurrTime);   // the function to get the time from the RTC

  if (timeStatus() != timeSet) {
    Serial.println("Unable to sync with the RTC");
  } else {
    Serial.println("RTC has set new time.");
    Serial.println("Press 's' key to set.");

    time_t t = makeTime(tmNewTime);  // convert to Unix time.

    while (1) {
      while(!Serial.available()) {;}  // key wait

      int key = Serial.read();
      if (key == 's' || key == 'S') {  // 'S'キーが押された
        RTC.setCurrTime(tmNewTime);    // DS1337 に新しい日時を書き込む」
        setTime(t);
        break;
      }
    }
  }
}

void loop()
{
  digitalClockDisplay();  // 1秒ごとにシリアルモニタに日時を転送する
  delay(1000);
}

void digitalClockDisplay(){
  // digital clock display of the time
  Serial.print(hour());
  printDigits(minute());
  printDigits(second());
  Serial.print(" ");
  Serial.print(day());
  Serial.print("/");
  Serial.print(monthShortStr(month()));  // 月は JAN とかで表示する
  Serial.print("/");
  Serial.print(year());
  Serial.print(" (");
  Serial.print(dayShortStr(weekday()));  // 曜日は SUN とかで表示する
  Serial.println(")");
}

void printDigits(int digits){
  // utility function for digital clock display: prints preceding colon and leading 0
  Serial.print(":");
  if(digits < 10)
    Serial.print('0');
  Serial.print(digits);
}

