/*
 * Print clock and battery capacity for Wakeup!!xxduino!!shield
 *
 * 目覚めよ！xxduino！シールドの DS1337 の現在の日時と、電池の現在の電圧を、シリアルモニターに転送するスケッチ例
 *  注意：電圧値はあくまでも目安です。絶対視しないでください。
 *
 * Copyright (C) 2013 I.Kato @ BS21Lab.
 *
 * The sketch is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This sketch is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this sketch; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "wiring_private.h"
#include "pins_arduino.h"

#include <Time.h>
#include <Wire.h>
#include <DS1337RTC.h>  // a basic DS1337 library that returns time as a time_t


#define WAKEUPUINO_VOUT_MON_PIN  A0   // 目覚めよ！xxduino！シールドの Vout モニター用のアナログ入力ピン: A0 か A3 のどちらか


void setup()
{
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  RTC.powerOffWithoutWaking();  // 目覚めよ！xxduino！シールドのアラーム１出力を強制 OFF します。単四電池の供給が停止するので、USB 接続されていないと Arduino も停止します。

  // Time ライブラリに DS1337 の getCurrTime 関数を渡します。以降、Time ライブラリ内で DS1337 の日時が取得されます。
  setSyncProvider(RTC.getCurrTime);   // the function to get the time from the RTC

  if (timeStatus() != timeSet)
    Serial.println("Unable to sync with the RTC");
  else
    Serial.println("RTC has set the system time");
}

void loop()
{
  // 目覚めよ！xxduino！シールドの SS1 スイッチを Cont.側にすると、単四電池６本の電圧値が表示されます。
  // SS1 スイッチを RTC 側にすると USB 供給の電圧値（5V-α）が表示されます。(αはレギュレーターの電圧降下の分)
  // 実験では、テスターでは 7.09V の電池が、このスケッチでは 7.25V と大きめに表示されましたので、数％の誤差がある事になります。

  digitalClockAndBattCapDisplay();   // 1秒ごとにシリアルモニタに日時と電池電圧を転送する
  delay(1000);
}

void digitalClockAndBattCapDisplay()
{
  // digital clock display of the time
  Serial.print(hour());
  printDigits(minute());
  printDigits(second());
  Serial.print(" ");
  Serial.print(day());
  Serial.print("/");
  Serial.print(monthShortStr(month()));  // 月は JAN とかで表示する
  Serial.print("/");
  Serial.print(year());
  Serial.print(" (");
  Serial.print(dayShortStr(weekday()));  // 曜日は SUN とかで表示する
  Serial.print(") : Vout=");

  int vom = analogRead (WAKEUPUINO_VOUT_MON_PIN);  // Voutモニター値(0～1023の範囲)の読み取り
  int vbg = analogReadBandgap ();  // Bandgap電圧(約1.1V)を読み出す：仕様上は 1.0V～1.2V の範囲なので、あくまで目安。
  float vout = (1.1 / vbg) * vom * 2.5;  // Bandgap電圧が約1.1V である事を利用して、Vout電圧を計算する。2.5倍しているのは、Vout を約2/5 に抵抗分圧している為。

//  vout = vout * 1.0;  // 線形補正させる：実機で実測して、1.0 の部分を適宜修正して下さい。

  Serial.print(vout);   // Vout電圧値を表示する。補正していない場合は、測定誤差などで小数点第一位以下は目安なので、値を絶対視しない事。
  Serial.print("V/vom=");
  Serial.print(vom);  // debug用
  Serial.print("/vbg=");
  Serial.print(vbg);  // debug用
  Serial.print("\n");
}

void printDigits(int digits)
{
  // utility function for digital clock display: prints preceding colon and leading 0
  Serial.print(":");
  if (digits < 10)
    Serial.print('0');
  Serial.print(digits);
}



// Bandgap電圧(約1.1V：仕様上は1.0V～1.2Vの範囲)を読み出す

extern uint8_t analog_reference;

int analogReadBandgap (void)
{
	uint8_t low, high;

#if defined(ADCSRB) && defined(MUX5)
	ADCSRB = (ADCSRB & ~(1 << MUX5));
#endif

	// ADMUX レジスタの MUX で Bandgap電圧を選択する
	// Uno(m328P) と Leonardo(m32u4) で動作確認。m1280/2560 と t25/45/85 は未確認。
#if defined(ADMUX)
#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
	ADMUX = (analog_reference << 6) | B11110;
#elif defined(__AVR_ATmega32U4__)
	ADMUX = (analog_reference << 6) | B11110;
#elif defined(__AVR_ATmega1284__) || defined(__AVR_ATmega1284P__) || defined(__AVR_ATmega644__) || defined(__AVR_ATmega644A__) || defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644PA__)
	ADMUX = (analog_reference << 6) | B11110;
#elif defined(__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
	// After switching to internal voltage reference the ADC requires a settling time of 1ms before measurements are stable.
	// Conversions starting before this may not be reliable. The ADC must be enabled during the settling time.
	ADMUX = (analog_reference << 6) | B1100;
#else
	ADMUX = (analog_reference << 6) | B1110;  // m88/m168/m328
#endif
#endif

	// without a delay, we seem to read from the wrong channel
	delay(1);  // 待ちが無いと、読み値がおかしくなる

#if defined(ADCSRA) && defined(ADCL)
	// start the conversion
	sbi(ADCSRA, ADSC);

	// ADSC is cleared when the conversion finishes
	while (bit_is_set(ADCSRA, ADSC));

	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low  = ADCL;
	high = ADCH;
#else
	// we dont have an ADC, return 0
	low  = 0;
	high = 0;
#endif

	// combine the two bytes
	return (high << 8) | low;  // Bandgap電圧(12bit:0～1023の範囲。Uno(m328P) の場合、大体 220 付近だった)
}

